#include "wiiuse.h"
#include "actionset.h"
#include <cstdlib>
#include <fstream>

using namespace std;

extern wiimote** wiimotes;
extern bool WIIMOTE_IR_MOUSE;
extern bool WIIMOTE_ACC_MOUSE;
extern ActionSet actions;
extern CC_ActionSet ccactions;

ActionSet::ActionSet()
{
name = "";
a = "";
b = "";
one = "";
two = "";
plus = "";
minus = "";
up = "";
down = "";
left = "";
right = "";
home = "";
return;
}

ActionSet::ActionSet(istream &actionfile)
{
name = "null profile";
a = "";
b = "";
one = "";
two = "";
plus = "";
minus = "";
up = "";
down = "";
left = "";
right = "";
home = "";

WIIMOTE_ACC_MOUSE = false; // will only get set to true if it is in this profile
WIIMOTE_IR_MOUSE = false; // will only get set to true if it is in this profile

string taglist[] = {"title", "a", "b", "1", "2", "home", "plus", 
                    "minus", "up", "down", "left", "right", "flick",
                    "ir_mouse", "acc_mouse"};
int numtags = 15;

string someline, value;
while (!actionfile.eof())
    {
    getline(actionfile, someline);
    for (int i=0;i < numtags;i++)
        {
        if (someline.find("["+taglist[i]+"]",0) != string::npos &&
            someline.find("[/"+taglist[i]+"]",0) != string::npos)
            {
            /* cout << taglist[i] << " tag found." << endl
                 << "Tag Length: " << taglist[i].length() << endl; */
            string::size_type start = someline.find("["+taglist[i]+"]",0)+taglist[i].length()+2,
                              end   = someline.find("[/"+taglist[i]+"]",0) - start;
            value = someline.substr(start,end);
            /* cout << "Extrated Value: " << value << endl; */
            switch (i)
                {
                case 0: // title tag
                    name = value;
                    break;
                case 1: // a tag
                    a = value;
                    break;
                case 2: // b tag
                    b = value;
                    break;
                case 3: // 1 tag
                    one = value;
                    break;
                case 4: // 2 tag
                    two = value;
                    break;
                case 5: // home tag
                    home = value;
                    break;
                case 6: // plus tag
                    plus = value;
                    break;
                case 7: // minus tag
                    minus = value;
                    break;
                case 8: // up tag
                    up = value;
                    break;
                case 9: // down tag
                    down = value;
                    break;
                case 10: // left tag
                    left = value;
                    break;
                case 11: // right tag
                    right = value;
                    break;
                case 12: // flick tag
                    flick = value;
                    break;
                case 13: // ir_mouse tag
                    if (value.find("x",0) != string::npos)
                    {
                        int x_res, y_res;
                        start = 0;
                        end = value.find("x",0);
                        x_res = atoi(value.substr(start,end).c_str());
                        start = end+1;
                        y_res = atoi(value.substr(start).c_str());
                        wiiuse_set_ir(wiimotes[0], 1);
                        wiiuse_set_ir_vres(wiimotes[0], x_res, y_res);
                        WIIMOTE_IR_MOUSE = true;
                        cout << "IR Mouse enabled" << endl;
                    }
                    else
                    {
                        cout << "Bad screen resolution specified in [ir_mouse] tag." << endl;
                    }
                    break;
                case 14: // acc_mouse tag
                    if (atoi(value.c_str()) == 1)                
                        WIIMOTE_ACC_MOUSE = true;
                        cout << "ACC Mouse enabled" << endl;
                    break;
                default:
                    break;
                }
                if (!WIIMOTE_IR_MOUSE)
                    {
                    wiiuse_set_ir(wiimotes[0], 0);
                    }
            }
        }
    }
return;
}

CC_ActionSet::CC_ActionSet()
{
name = "null profile";
a = "";
b = "";
x = "";
y = "";
plus = "";
minus = "";
d_up = "";
d_down = "";
d_left = "";
d_right = "";
home = "";
l_part = "";
r_part = "";
l = "";
r = "";
zl = "";
zr = "";
ls_up = "";
ls_down = "";
ls_left = "";
ls_right = "";
rs_up = "";
rs_down = "";
rs_left = "";
rs_right = "";
return;
}

CC_ActionSet::CC_ActionSet(istream &actionfile)
{
name = "null profile";
a = "";
b = "";
x = "";
y = "";
plus = "";
minus = "";
d_up = "";
d_down = "";
d_left = "";
d_right = "";
home = "";
l_part = "";
r_part = "";
l = "";
r = "";
zl = "";
zr = "";
ls_up = "";
ls_down = "";
ls_left = "";
ls_right = "";
rs_up = "";
rs_down = "";
rs_left = "";
rs_right = "";
string taglist[] = {"title", "cc_a", "cc_b", "cc_x", "cc_y", "cc_home", "cc_plus", "cc_minus", 
                    "cc_dpad_up", "cc_dpad_down", "cc_dpad_left", "cc_dpad_right",
                    "cc_zl", "cc_zr", "cc_l", "cc_r",
                    "cc_ls_up", "cc_ls_down", "cc_ls_left", "cc_ls_right",
                    "cc_rs_up", "cc_rs_down", "cc_rs_left", "cc_rs_right",
                    "cc_l_part", "cc_r_part" };
int numtags = 26;

string someline, value;
while (!actionfile.eof())
    {
    getline(actionfile, someline);
    for (int i=0;i < numtags;i++)
        {
        if (someline.find("["+taglist[i]+"]",0) != string::npos &&
            someline.find("[/"+taglist[i]+"]",0) != string::npos)
            {
            /*cout << taglist[i] << " tag found." << endl
                 << "Tag Length: " << taglist[i].length() << endl; */
            string::size_type start = someline.find("["+taglist[i]+"]",0)+taglist[i].length()+2,
                              end   = someline.find("[/"+taglist[i]+"]",0) - start;
            value = someline.substr(start,end);
            /* cout << "Extrated Value: " << value << endl; */
            switch (i)
                {
                case 0: // title tag
                    name = value;
                    break;
                case 1: // cc_a tag
                    a = value;
                    break;
                case 2: // cc_b tag
                    b = value;
                    break;
                case 3: // cc_x tag
                    x = value;
                    break;
                case 4: // cc_y tag
                    y = value;
                    break;
                case 5: // cc_home tag
                    home = value;
                    break;
                case 6: // cc_plus tag
                    plus = value;
                    break;
                case 7: // cc_minus tag
                    minus = value;
                    break;
                case 8: // cc_dpad_up tag
                    d_up = value;
                    break;
                case 9: // cc_dpad_down tag
                    d_down = value;
                    break;
                case 10: // cc_dpad_left tag
                    d_left = value;
                    break;
                case 11: // cc_dpad_right tag
                    d_right = value;
                    break;
                case 12: // cc_zl tag
                    zl = value;
                    break;
                case 13: // cc_zr tag
                    zr = value;
                    break;
                case 14: // cc_l tag
                    l = value;
                    break;
                case 15: // cc_r tag
                    r = value;
                    break;
                case 16: // cc_ls_up tag
                    ls_up = value;
                    break;
                case 17: // cc_ls_down tag
                    ls_down = value;
                    break;
                case 18: // cc_ls_left tag
                    ls_left = value;
                    break;
                case 19: // cc_ls_right tag
                    ls_right = value;
                    break;
                case 20: // cc_rs_up tag
                    rs_up = value;
                    break;
                case 21: // cc_rs_down tag
                    rs_down = value;
                    break;
                case 22: // cc_rs_left tag
                    rs_left = value;
                    break;
                case 23: // cc_rs_right tag
                    rs_right = value;
                    break;
                case 24: // cc_l_part tag
                    l_part = value;
                    break;
                case 25: // cc_r_part tag
                    r_part = value;
                    break;
                }
            }
        }
    }
return;
}

void execute_action(string event, stateType type)
{
string::size_type loc = event.find(" ");
string command = event.substr(0, loc);
if (command == "KEYBOARD")
    {
    string keystr = event.substr(loc+1);
    KeyCode keycode = get_x_keycode_from_string(keystr);
        switch (type)
            {
            case RELEASED:
                XIUseKeyboard(keycode, true);
                break;
            case PRESSED:
                XIUseKeyboard(keycode, false);
                break;
            default:
                break;
            }
    }
else if (command == "MOUSECLICK")
    {
    string keystr = event.substr(loc+1);
    int btncode = atoi(keystr.c_str());
        switch (type)
            {
            case RELEASED:
                XIClickMouse(btncode, true);
                break;
            case PRESSED:
                XIClickMouse(btncode, false);
                break;
            default:
                break;
            }
    }
else if (command == "SWITCHPROFILE" && type == PRESSED)
    {
    string keystr = event.substr(loc+1);
    ifstream profile;
    profile.open(keystr.c_str(), ios::in);
    if (!profile.is_open())
    {
    cout << "Could not open specified config file for reading.  Exiting..." << endl;
    exit(1);
    }
    actions = ActionSet(profile);
    profile.close();
    profile.open(keystr.c_str(), ios::in);
    if (!profile.is_open())
    {
    cout << "Could not open specified config file for reading.  Exiting..." << endl;
    exit(1);
    }
    ccactions = CC_ActionSet(profile);
    profile.close();
    }
else if (type == PRESSED) // just execute the string as a command
    {
    event += " > /dev/null";  // who cares about the output.  Change this if you do.
    system(event.c_str());
    }
return;
}

string ActionSet::get_name()
{
return name;
}

string CC_ActionSet::get_name()
{
return name;
}

InputState::InputState()
{
a = false;
b = false;
one = false;
two = false;
plus = false;
minus = false;
up = false;
down = false;
left = false;
right = false;
home = false;
flick = false;
a_type = NO_CHANGE;
b_type = NO_CHANGE;
one_type = NO_CHANGE;
two_type = NO_CHANGE;
plus_type = NO_CHANGE;
minus_type = NO_CHANGE;
up_type = NO_CHANGE;
down_type = NO_CHANGE;
left_type = NO_CHANGE;
right_type = NO_CHANGE;
home_type = NO_CHANGE;
flick_type = NO_CHANGE;
}

CC_InputState::CC_InputState()
{
a = false;
b = false;
x = false;
y = false;
plus = false;
minus = false;
home = false;
d_up = false;
d_down = false;
d_left = false;
d_right = false;
l_part = false;
r_part = false;
l = false;
r = false;
zl = false;
zr = false;
ls_up = false;
ls_down = false;
ls_left = false;
ls_right = false;
rs_up = false;
rs_down = false;
rs_left = false;
rs_right = false;

a_type = NO_CHANGE;
b_type = NO_CHANGE;
x_type = NO_CHANGE;
y_type = NO_CHANGE;
plus_type = NO_CHANGE;
minus_type = NO_CHANGE;
home_type = NO_CHANGE;
d_up_type = NO_CHANGE;
d_down_type = NO_CHANGE;
d_left_type = NO_CHANGE;
d_right_type = NO_CHANGE;
l_part_type = NO_CHANGE;
r_part_type = NO_CHANGE;
l_type = NO_CHANGE;
r_type = NO_CHANGE;
zl_type = NO_CHANGE;
zr_type = NO_CHANGE;
ls_up_type = NO_CHANGE;
ls_down_type = NO_CHANGE;
ls_left_type = NO_CHANGE;
ls_right_type = NO_CHANGE;
rs_up_type = NO_CHANGE;
rs_down_type = NO_CHANGE;
rs_left_type = NO_CHANGE;
rs_right_type = NO_CHANGE;
}


void xwii_process_input(InputState &istate, ActionSet &actions)
{
if (istate.a)
    {
    execute_action(actions.a, istate.a_type);
    }
if (istate.b)
    {
    execute_action(actions.b, istate.b_type);
    }
if (istate.one)
    {
    execute_action(actions.one, istate.one_type);
    }
if (istate.two)
    {
    execute_action(actions.two, istate.two_type);
    }
if (istate.plus)
    {
    execute_action(actions.plus, istate.plus_type);
    }
if (istate.minus)
    {
    execute_action(actions.minus, istate.minus_type);
    }
if (istate.up)
    {
    execute_action(actions.up, istate.up_type);
    }
if (istate.down)
    {
    execute_action(actions.down, istate.down_type);
    }
if (istate.left)
    {
    execute_action(actions.left, istate.left_type);
    }
if (istate.right)
    {
    execute_action(actions.right, istate.right_type);
    }
if (istate.home)
    {
    execute_action(actions.home, istate.home_type);
    }
if (istate.flick)
    {
    execute_action(actions.flick, istate.flick_type);
    }
return;
}

void xwii_process_CC_input(CC_InputState &istate, CC_ActionSet &actions)
{
if (istate.a)
    {
    execute_action(actions.a, istate.a_type);
    }
if (istate.b)
    {
    execute_action(actions.b, istate.b_type);
    }
if (istate.x)
    {
    execute_action(actions.x, istate.x_type);
    }
if (istate.y)
    {
    execute_action(actions.y, istate.y_type);
    }
if (istate.plus)
    {
    execute_action(actions.plus, istate.plus_type);
    }
if (istate.minus)
    {
    execute_action(actions.minus, istate.minus_type);
    }
if (istate.d_up)
    {
    execute_action(actions.d_up, istate.d_up_type);
    }
if (istate.d_down)
    {
    execute_action(actions.d_down, istate.d_down_type);
    }
if (istate.d_left)
    {
    execute_action(actions.d_left, istate.d_left_type);
    }
if (istate.d_right)
    {
    execute_action(actions.d_right, istate.d_right_type);
    }
if (istate.home)
    {
    execute_action(actions.home, istate.home_type);
    }
if (istate.l)
    {
    execute_action(actions.l, istate.l_type);
    }
if (istate.r)
    {
    execute_action(actions.r, istate.r_type);
    }
if (istate.l_part)
    {
    execute_action(actions.l_part, istate.l_part_type);
    }
if (istate.r_part)
    {
    execute_action(actions.r_part, istate.r_part_type);
    }
if (istate.zr)
    {
    execute_action(actions.zr, istate.zr_type);
    }
if (istate.zl)
    {
    execute_action(actions.zl, istate.zl_type);
    }
if (istate.ls_up)
    {
    execute_action(actions.ls_up, istate.ls_up_type);
    }
if (istate.ls_down)
    {
    execute_action(actions.ls_down, istate.ls_down_type);
    }
if (istate.ls_left)
    {
    execute_action(actions.ls_left, istate.ls_left_type);
    }
if (istate.ls_right)
    {
    execute_action(actions.ls_right, istate.ls_right_type);
    }
if (istate.rs_up)
    {
    execute_action(actions.rs_up, istate.rs_up_type);
    }
if (istate.rs_down)
    {
    execute_action(actions.rs_down, istate.rs_down_type);
    }
if (istate.rs_left)
    {
    execute_action(actions.rs_left, istate.rs_left_type);
    }
if (istate.rs_right)
    {
    execute_action(actions.rs_right, istate.rs_right_type);
    }
}
